# Copyright 2016-2018 California Institute of Technology.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import requests
from future.moves.urllib.request import urlretrieve
from future.moves.urllib.parse import urlparse
import os

SEARCH_URL = 'https://oceandata.sci.gsfc.nasa.gov/api/file_search?'
#GET_URL = 'https://oceandata.sci.gsfc.nasa.gov/cgi/getfile/'
HEADERS = {
    'User-Agent': 'Podaacpy Python Library'
}

class OceanColor:

    def __init__(self):
        self.SEARCH_URL = 'https://oceandata.sci.gsfc.nasa.gov/api/file_search?'
        #self.GET_URL = 'https://oceandata.sci.gsfc.nasa.gov/cgi/getfile/'

    def file_search(self, sensor='', sdate='', edate='', dtype='', add_url='1', results_as_file='1',
                       search='', sub_id='', std_only='1', cksum='', output_format='json'):
        '''File search service retrieves publically available files within the \
                NASA Ocean Data Processing System.

        :param sensor: mission name. valid options include: aquarius, seawifs, \
                aqua, terra, meris, octs, czcs, hico, viirs
        :type sensor: :mod:`string`

        :param sdate: start date for a search
        :type sdate: :mod:`string`

        :param edate: end date for a search
        :type edate: :mod:`string`

        :param dtype: data type (i.e. level). valid options: L0, L1, L2, L3b \
                (for binned data), L3m (for mapped data), MET (for ancillary \
                data), misc (for sundry products)
        :type dtype: :mod:`string`

        :param add_url: include full url in search result (boolean, \
                1=yes, 0=no)
        :type add_url: :mod:`string`

        :param results_as_file: return results as a test file \
                listing (boolean, 1=yes, 0=no, thus returns and HTML page)
        :type results_as_file: :mod:`string`

        :param search: text string search
        :type search: :mod:`string`

        :param sub_id: non-extracted subscription ID to search
        :type sub_id: :mod:`string`

        :param std_only: restrict results to standard products \
                (i.e. ignore extracts, regional processings, etc.; boolean)
        :type std_only: :mod:`string`

        :param cksum: return a checksum file for search results \
                (boolean; sha1sums except for Aquarius soil moisture \
                products which are md5sums; forces results_as_file; ignores addurl)
        :type cksum: :mod:`string`

        :param output_format: valid options are: 'json', 'txt' and 'html'
        :type output_format: :mod:`string`

        :returns: by default a json response based on the requested 'output_format'. Options \
        are 'json, 'txt' and 'html'.

        '''
        try:
            url = SEARCH_URL
            if sensor:
                url = url + 'sensor=' + sensor
            else:
                raise Exception("'sensor' parameter is required!")
            if sdate:
                url = url + '&sdate=' + sdate
            if edate:
                url = url + '&edate=' + edate
            if dtype:
                url = url + '&dtype=' + dtype
            url = url + '&addurl=' + str(add_url)
            url = url + '&results_as_file=' + str(results_as_file)
            if search:
                url = url + '&search=' + search
            elif sub_id:
                url = url + '&subID=' + sub_id
            else:
                raise Exception("Either 'search' or 'sub_id' parameter is required!")
            url = url + '&std_only=' + str(std_only)
            if cksum:
                url = url + '&cksum=' + cksum
            url = url + '&format=' + output_format

            response = requests.post(url, headers=HEADERS)
            status_codes = [404, 400, 503, 408]
            if response.status_code in status_codes:
                response.raise_for_status()

        except requests.exceptions.HTTPError as error:
            print(error)
            raise

        return response.text

    def get_file(self, url='', path=''):
        '''It is possible to mimic FTP bulk data downloads using the \
                HTTP-based data distribution server at https://oceandata.sci.gsfc.nasa.gov.

        :param url: a single file name which can be obtained by calling #file_search() \
                an example would be \
                https://oceandata.sci.gsfc.nasa.gov/cgi/getfile/O1997001.L3b_DAY_CHL.nc
        :type url: :mod:`string`

        :param path: Destination directory into which the granule \
                needs to be downloaded.
        :type path: :mod:`string`

        :returns: a file object downloaded from the \
                HTTP-based data distribution server at https://oceandata.sci.gsfc.nasa.gov.

        '''
        try:
            #url = GET_URL
            if url:
                url = url
            else:
                raise Exception("'file' parameter is required!")
            file = os.path.basename(urlparse(url).path)
            if path == '':
            	path = os.path.join(os.path.dirname(__file__), file)
            else:
                path = path + '/' + file
            urlretrieve(url, path)
            print("Downloaded '%s' to '%s'" % (file, path))
            return file

        except Exception:
            raise



# Copyright 2016-2018 California Institute of Technology.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ..oceancolor import OceanColor
import os
import requests
import xml.etree.ElementTree as ET
from nose.tools import assert_raises
import unittest
from future.moves.urllib.error import HTTPError


class TestOceanColor(unittest.TestCase):

    @classmethod
    def setUp(self):
        self.oceancolor = OceanColor()

    # test case for the function file_search()
    def test_file_search(self):
        data = self.oceancolor.file_search(sensor='octs', sdate='1996-11-01', edate='1997-01-01',
                dtype='L3b', add_url='1', results_as_file='1', search='*DAY_CHL*')

        assert data != None
        print(data)
        assert type(data) is type(u'')
        assert len(data) != 0

        assert_raises(Exception, self.oceancolor.file_search, sensor='random')
        assert_raises(Exception, self.oceancolor.file_search, sdate='1996-11-01', edate='1997-01-01',
                dtype='L3b', add_url='1', results_as_file='1', search='*DAY_CHL*')

    # test case for the function get_file(()
    def test_get_file(self):
        url = 'https://oceandata.sci.gsfc.nasa.gov/cgi/getfile/O1996307.L3b_DAY_CHL.nc'
        path = os.path.dirname(os.path.abspath(__file__))
        granule_name = self.oceancolor.get_file(url, path)

        assert granule_name != None
        assert_raises(Exception, self.oceancolor.get_file,
                      url='ABCDEF')

        path = os.path.join(os.path.dirname(__file__), granule_name)
        os.remove(path)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
import json
import requests
import os
from future.moves.urllib.error import HTTPError
from ..l2ss import L2SS
from nose.tools import assert_raises


class TestL2SS(unittest.TestCase):

    @classmethod
    def setUp(self):
        self.l2ss = L2SS()

    # test case for the fucntion dataset_search()
    def test_dataset_search(self):
        dataset_id = 'PODAAC-ASOP2-25X01'
        variable = ['Surface Winds']
        sensor = ['Advanced Scatterometer']
        provider = ['KNMI']
        start_time = '2016-12-4T22:39:52Z'
        start_index = '0'
        items_per_page = '7'
        dataset = self.l2ss.dataset_search(dataset_id=dataset_id, variable=variable, start_time=start_time, sensor=sensor,
                                           provider=provider, start_index=start_index, items_per_page=items_per_page)
        dataset_json = json.loads(dataset)

        assert dataset_json['response']['docs'][0][
            'Dataset-PersistentId'] == dataset_id

    # test case for the function dataset_variables()
    def test_dataset_variables(self):
        dataset_id = 'PODAAC-ASOP2-25X01'
        variables = json.loads(
            self.l2ss.dataset_variables(dataset_id=dataset_id))
        variables_data = variables['imgVariables']

        assert len(variables_data) != 0
        assert_raises(requests.exceptions.HTTPError,
                      self.l2ss.dataset_variables, dataset_id='PODAAC')

    # test case for the function granule_search
    def test_granule_search(self):
        dataset_id = 'PODAAC-ASOP2-25X01'
        bbox = '-180,-90,180,90'
        start_time = '2016-07-16T04:18:00Z'
        end_time = '2016-07-16T05:56:56Z'
        items_per_page = '7'
        start_index = '0'
        name = 'ascat_20160716_041800_metopa_50541_eps_o_250_2401_ovw.l2.nc'
        sort = 'Granule-Name asc'
        granules = self.l2ss.granule_search(dataset_id=dataset_id, bbox=bbox, start_time=start_time, end_time=end_time,
                                            items_per_page=items_per_page, start_index=start_index, name=name, sort=sort)
        granules_json = json.loads(granules)

        assert granules_json['response']['docs'][0][
            'Granule-DatasetId'] == dataset_id

    # test case for the function granules_availability
    def test_granules_availability(self):
        dataset_id = 'PODAAC-ASOP2-25X01'
        bbox = '-180,-90,180,90'
        start_time = '2014-10-12T11:42:00Z'
        end_time = '2016-10-12T11:42:00Z'
        gap = 'DAY'
        granule_availability = json.loads(self.l2ss.granules_availability(
            dataset_id=dataset_id, start_time=start_time, end_time=end_time, bbox=bbox, gap=gap))
        availability_data = granule_availability[
            'facet_counts']['facet_dates']['Granule-StartTime']

        assert len(availability_data) != 0
        assert_raises(requests.exceptions.HTTPError,
                      self.l2ss.granules_availability, dataset_id=dataset_id)

    # test case for the function granule_preview_image
    def test_granule_preview_image(self):
        dataset_id = 'PODAAC-ASOP2-25X01'
        granule = 'ascat_20140520_005700_metopa_39344_eps_o_250_2300_ovw.l2.nc'
        year = '2014'
        day = '140'
        variable = 'wind_speed'
        path = os.path.join(os.path.dirname(__file__))
        image = self.l2ss.granule_preview_image(
            dataset_id=dataset_id, granule=granule, year=year, day=day, variable=variable, path=path)

        assert image != None
        path = os.path.join(os.path.dirname(__file__),
                            dataset_id + '.png')
        os.remove(path)
        assert_raises(HTTPError,
                      self.l2ss.granule_preview_image, dataset_id=dataset_id, granule=granule, year=year, day=day, variable='wind_spee')

    # test case for the function image_palette
    def test_image_palette(self):
        palette_name = 'paletteMedspirationIndexed'
        test_palette_name = 'medspiration'
        palette = json.loads(
            self.l2ss.image_palette(palette_name=palette_name))
        palette_name = palette['Palette'][
            'attributes']['attribute'][0]['value']

        assert palette_name == test_palette_name
        assert_raises(requests.exceptions.HTTPError,
                      self.l2ss.image_palette, palette_name='SomeUnknownPalette')

    # test case for the function granule_download
    def test_granule_download(self):
        query = {
            "email": "unknown@unknown.com",
            "query":
            [
                {
                    "compact": "true",
                    "datasetId": "PODAAC-ASOP2-25X01",
                    "bbox": "-180,-90,180,90",
                    "variables": ["lat", "lon", "time", "wind_speed"],
                    "granuleIds": ["ascat_20140520_005700_metopa_39344_eps_o_250_2300_ovw.l2.nc"]
                }
            ]
        }
        self.l2ss.granule_download(query_string=query)
        assert os.path.isfile(
            './subsetted-ascat_20140520_005700_metopa_39344_eps_o_250_2300_ovw.l2.nc') == True
        os.remove(
            './subsetted-ascat_20140520_005700_metopa_39344_eps_o_250_2300_ovw.l2.nc')

    # test case for the function subset_status
    def test_subset_status(self):
        token = 'FakeToken'
        assert_raises(Exception, self.l2ss.subset_status, token=token)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import requests
import os
import json
import time
import zipfile
from future.moves.urllib.request import urlopen, urlretrieve
from future.moves.urllib.parse import urlencode
from future.moves.http.client import HTTPSConnection


class L2SS:

    def __init__(self):
        self.URL = 'https://podaac-tools.jpl.nasa.gov/l2ss-services/l2ss/'

    def dataset_search(self, dataset_id='', variable=[], sensor=[], provider=[], start_time='', end_time='', start_index='', items_per_page=''):
        ''' Dataset search service lists available datasets and returns them.

            :param dataset_id: Search dataset belong to given PODAAC Dataset persistent ID.
            :type dataset_id: :mod:`string`

            :param variable: Search for datasets with variable name. For multi-value input, \
                this input is taken as a list. Example: [ 'Sea Surface Temperature', 'Surface Wind']
            :type variable: :mod:`list`

            :param sensor: Search for datasets with sensor. For multi-value input, \
                this input is taken as a list.
            :type sensor: :mod:`list`

            :param provider: Search for datasets with provider. For multi-value input, \
                this input is taken as a list.
            :type provider: :mod:`list`

            :param start_time: Lower time bound. If not specified, lower time bound of \
                the dataset will be used. Example: '2011-12-31T23:59:59-06:00Z'
            :type start_time: :mod:`string`

            :param end_time: Upper time bound. If not specified, upper time bound of \
                the dataset will be used. Example: 2019-12-31T23:59:59-06:00Z
            :type end_time: :mod:`string`

            :param items_per_page: number of results to return.
            :type items_per_page: :mod:`string`

            :param start_index: start index of result.
            :type start_index: :mod:`string`

            :returns: a json response containing the datasets.
        '''
        try:
            url = self.URL + 'dataset/search?'
            if dataset_id:
                url = url + 'datasetId=' + dataset_id
            if variable:
                for var in variable:
                    url = url + '&variable=' + var
            if sensor:
                for item in sensor:
                    url = url + '&sensor=' + item
            if provider:
                for item in provider:
                    url = url + '&provider=' + item
            if start_time:
                url = url + '&startTime=' + start_time
            if end_time:
                url = url + '&endTime=' + end_time
            if start_index:
                url = url + '&startIndex=' + start_index
            if items_per_page:
                url = url + '&itemsPerPage=' + items_per_page

            datasets = requests.get(url)
            status_codes = [404, 400, 503, 408]
            if datasets.status_code in status_codes:
                datasets.raise_for_status()

        except requests.exceptions.HTTPError as error:
            print(error)
            raise

        return datasets.text

    def dataset_variables(self, dataset_id):
        ''' Dataset Variable retrieves dataset configuration information including variables.

            :param dataset_id: datasetId for the configuration information.
            :type dataset_id: :mod:`string`

            :returns: a json response containing the dataset variables.
        '''
        try:
            url = self.URL + '/dataset/variable?datasetId=' + dataset_id
            variables = requests.get(url)
            status_codes = [404, 400, 503, 408]
            if variables.status_code in status_codes:
                variables.raise_for_status()

        except requests.exceptions.HTTPError as error:
            print(error)
            raise

        return variables.text

    def granule_search(self, dataset_id='', bbox='', start_time='', end_time='', name='', sort='', start_index='', items_per_page=''):
        ''' Granule Search retrieves all base granule information (datasetId, start time, end time) \
            matching the specified datasetId, date, and region. This approach may change if \
            the data/querying turns out to be too expensive. Response is structured in a minimalistic\
            way to cut down on the file size.

            :param dataset_id: Search granules belong to given PODAAC Dataset persistent ID.
            :type dataset_id: :mod:`string`

            :param bbox: Search granules with Bounding box Ex: '-180,-90,180,90'
            :type bbox: :mod:`string`

            :param start_time: Lower time bound. If not specified, lower time bound of \
                the dataset will be used. Example: '2011-12-31T23:59:59-06:00Z'
            :type start_time: :mod:`string`

            :param end_time: Upper time bound. If not specified, upper time bound of \
                the dataset will be used. Example: 2019-12-31T23:59:59-06:00Z
            :type end_time: :mod:`string`

            :param name : Search granules with exact name or name pattern using wildcard\
                search Example: ascat* this matches name that starts with "ascat"
            :type name: :mod:`string`

            :param sort: Sort output. There are two strings delimited by space.\
                The first string is the field name, and the second string is 'asc' or 'desc'\
                Example: sort='Granule-Name asc'
            :type sort: :mod:`string`

            :param items_per_page: number of results to return.
            :type items_per_page: :mod:`string`

            :param start_index: start index of result.
            :type start_index: :mod:`string`

            :returns: a json response containing the dataset granules.
        '''
        try:
            url = self.URL + 'granule/search?'
            if dataset_id:
                url = url + 'datasetId=' + dataset_id
            if bbox:
                url = url + '&bbox=' + bbox
            if start_time:
                url = url + '&startTime=' + start_time
            if end_time:
                url = url + '&endTime=' + end_time
            if start_index:
                url = url + '&startIndex=' + start_index
            if items_per_page:
                url = url + '&itemsPerPage=' + items_per_page
            if name:
                url = url + '&name=' + name
            if sort:
                url = url + '&sort=' + sort

            granules = requests.get(url)
            status_codes = [404, 400, 503, 408]
            if granules.status_code in status_codes:
                granules.raise_for_status()

        except requests.exceptions.HTTPError as error:
            print(error)
            raise

        return granules.text

    def granules_availability(self, dataset_id='', start_time='', end_time='', gap='', bbox=''):
        ''' Granules Availability calculates granule counts per day or month from given date range.

            :param dataset_id: Search granules belong to given PODAAC Dataset persistent ID.
            :type dataset_id: :mod:`string`

            :param start_time: Lower time bound. If not specified, lower time bound of \
                the dataset will be used. Example: '2011-12-31T23:59:59-06:00Z'
            :type start_time: :mod:`string`

            :param end_time: Upper time bound. If not specified, upper time bound of \
                the dataset will be used. Example: 2019-12-31T23:59:59-06:00Z
            :type end_time: :mod:`string`

            :param gap: The size of each date range expressed as an interval to be added\
                to the lower bound. Example: 'DAY', 'MONTHS'
            :type gap: :mod:`string`

            :param bbox: Search granules with Bounding box Ex: '-180,-90,180,90'
            :type bbox: :mod:`string`

            :returns: a json response containing the granule count and other relevant information.
        '''
        try:
            url = self.URL + 'granule/availability?'
            url = url + 'datasetId=' + dataset_id + '&startTime=' + \
                start_time + '&endTime=' + end_time + '&gap=' + gap
            if bbox:
                url = url + '&bbox=' + bbox

            granule_availability = requests.get(url)
            status_codes = [404, 400, 503, 408]
            if granule_availability.status_code in status_codes:
                granule_availability.raise_for_status()

        except requests.exceptions.HTTPError as error:
            print(error)
            raise

        return granule_availability.text

    def granule_preview_image(self, dataset_id, granule, year, day, variable, path=''):
        ''' Granule Preview Image Service provides thumbnail image of selected variable for\
            selected granule.

            :param dataset_id: Search granules belong to given PODAAC Dataset persistent ID.
            :type dataset_id: :mod:`string`

            :param granule: string granule name.
            :type granule: :mod:`string`

            :param year: year in 4 digits. Example= '2014'
            :type year: :mod:`string`

            :param day: day of year in 3 digits. Example= '140'
            :type day: :mod:`string`

            :param variable_id: Variable id described in dataset variable service.
            :type variable_id: :mod:`string`

            :returns: returns thumbnail image of selected variable for selected granule.
        '''
        try:
            url = self.URL + 'preview/' + dataset_id + '/' + year + \
                '/' + day + '/' + granule + '/' + variable + '.png'
            if path:
                path = path + '/' + dataset_id + '.png'
            else:
                path = os.path.join(os.path.dirname(
                    __file__), dataset_id + '.png')
            image_file = open(path, 'wb')
            image = urlopen(url)
            image_file.write(image.read())

        except Exception:
            raise

        return image

    def image_palette(self, palette_name):
        ''' Image Palette service retrieves palette descriptor in json format

            :param palette_name: palette_name whose palette descriptor we want to\
                retrieve.
            :type palette_name: :mod:`string`

            :returns: returns palette descriptor in json format.
        '''
        try:
            url = self.URL + 'palettes/' + palette_name + '.json'
            image_palette = requests.get(url)
            status_codes = [404, 400, 503, 408]
            if image_palette.status_code in status_codes:
                image_palette.raise_for_status()

        except requests.exceptions.HTTPError as error:
            print(error)
            raise

        return image_palette.text

    def granule_download(self, query_string, path=''):
        ''' Granule Download service submits a job to subset and download. Upon a successful request,\
            token will be returned which can be used to check status.

            :param query_string: data collection query json as a string.
            :type query_string: :mod:`string`

            :param path: path to a directory where you want the subsetted \
                dataset to be stored.
            :type path: :mod:`string`

            :returns: a zip file downloaded and extracted in the destination\
                directory path provided.
        '''
        params = urlencode({'query': query_string})
        headers = {
            "Content-type": "application/x-www-form-urlencoded", "Accept": "*"}
        connection = HTTPSConnection("podaac-tools.jpl.nasa.gov")
        connection.request("POST", "/l2ss-services/l2ss/subset/submit",
                           params, headers)
        response = connection.getresponse()
        data = response.read().decode('utf-8')
        result = json.loads(data)
        token = result['token']
        connection.close()

        flag = 0
        while flag == 0:
            url = url = self.URL + "subset/status?token=" + token
            subset_response = requests.get(url).text
            subset_response_json = json.loads(subset_response)
            status = subset_response_json['status']
            if status == "done":
                flag = 1
            if status == "error":
                raise Exception(
                    "Unexpected error occured for the subset job you have requested")
            if status == "partial error":
                raise Exception(
                    "The job was done but with some errors, please submit the job again")
            time.sleep(1)

        print("Done! downloading the dataset zip .....")
        download_url = subset_response_json['resultURLs'][0]
        split = download_url.split('/')
        length = len(split)
        zip_file_name = split[length - 1]
        if path == '':
            path = os.path.join(os.path.dirname(__file__), zip_file_name)
        else:
            path = path + zip_file_name
        response = urlretrieve(download_url, path)
        zip_content = zipfile.ZipFile(path)
        zip_content.extractall()
        os.remove(path)

    def subset_status(self, token):
        ''' Subset Status service check status on existing download job.
            The possible status that it returns include the following.. ::

              * "queued"
              * "processing"
              * "partial error"
              * "done"
              * "error"

            :param token: job token. job token is provided when submitting the job.
            :type token: :mod:`string`

            :returns: the status of the subset request.
        '''
        try:
            url = self.URL + 'subset/status?token=' + token
            response = requests.get(url)
            response_json = json.loads(response.text)
            status = response_json['status']
            if status == "unknown":
                raise Exception("Invalid Token : Please check your token")

        except Exception:
            raise
